/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.ruleengine.expression;

import java.util.HashMap;
import java.util.Map;

import org.tinygroup.ruleengine.exception.MethodExecuteException;

/**
 * 自定义操作或Boolean表达式的父类
 * @author luoguo
 *
 * @param <T>
 */
public class CustomExpression<T> extends Expression<T> {
	public static final String REQUEST = "request";
	public static final String THREAD = "thread";
	public static final String SINGLETON = "singleton";

	private static Map<String, Object> objectCache = new HashMap<String, Object>();
	private static ThreadLocal<Map<String, Object>> objectCacheThreadLocal = new ThreadLocal<Map<String, Object>>();
	private String lifeCircle = SINGLETON;
	private String className;

	public String getLifeCircle() {
		return lifeCircle;
	}

	public void setLifeCircle(String lifeCircle) {
		this.lifeCircle = lifeCircle;
	}

	public String getName() {
		return "custom";
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public T evaluate() {
		String cname = className;
		Object o = null;
		Map<String, Object> ht = null;
		if (lifeCircle.equals(SINGLETON)) {
			ht = objectCache;
		} else if (lifeCircle.equals(THREAD)) {
			ht = objectCacheThreadLocal.get();
		}
		if (ht != null) {
			o = ht.get(cname);
		}
		if (o == null) {
			try {
				Class c = Class.forName(cname);
				o = c.newInstance();
				if (o instanceof IExpression) {
					IExpression e = (IExpression) o;
					e.setRuleSession(getRuleSession());
					T result = (T) e.evaluate();
					ht.put(cname, o);
					return result;
				}
			} catch (Exception e) {
				throw new MethodExecuteException(e);
			}
		}
		return null;
	}
}
