/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.ruleengine.expression.common;

import org.apache.commons.beanutils.MethodUtils;
import org.tinygroup.ruleengine.exception.MethodExecuteException;
import org.tinygroup.ruleengine.expression.Expression;

public class MethodActionExpression extends Expression<Object> {

	private String varName;
	private String method;
	private String args[];

	public String getName() {
		return "methodaction";
	}

	public String getVarName() {
		return varName;
	}

	public void setVarName(String varName) {
		this.varName = varName;
	}

	public String getMethod() {
		return method;
	}

	public void setMethod(String method) {
		this.method = method;
	}

	public String[] getArgs() {
		return args;
	}

	public void setArgs(String[] args) {
		this.args = args.clone();
	}

	public Object evaluate() {
		Object o = getObjectByName(varName);
		Object[] trueArgs = null;
		if (args != null) {
			trueArgs = new Object[args.length];
			for (int i = 0; i < args.length; i++) {
				trueArgs[i] = getObjectByName(args[i]);
			}
		}
		return executeMethod(o, method, trueArgs);
	}

	private Object executeMethod(Object o, String method, Object[] trueArgs) {
		try {
			return MethodUtils.invokeMethod(o, method, trueArgs);
		} catch (Exception e) {
			throw new MethodExecuteException(e);
		}
	}
}
