/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.ruleengine.expression.decisiontable;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.tinygroup.ruleengine.expression.Expression;

public class DecisionTable extends Expression<Object> {
	// 只有所有的条件都满足才可以
	private List<DecisionTableConditionItem> decisionItems;
	// 所有可能被执行的操作
	private List<DecisionTableActionItem> actionItems;
	// 值表
	private List<DecisionTableValue> valueTable;

	public void addDecisionTableConditionItem(DecisionTableConditionItem item) {
		if (decisionItems == null) {
			decisionItems = new ArrayList<DecisionTableConditionItem>();
		}
		decisionItems.add(item);
	}

	public void addDecisionTableActionItem(DecisionTableActionItem item) {
		if (actionItems == null) {
			actionItems = new ArrayList<DecisionTableActionItem>();
		}
		actionItems.add(item);
	}

	public void addDecisionTableValue(DecisionTableValue value) {
		if (valueTable == null) {
			valueTable = new ArrayList<DecisionTableValue>();
		}
		valueTable.add(value);
	}

	public List<DecisionTableConditionItem> getDecisionItems() {
		return decisionItems;
	}

	public void setDecisionItems(List<DecisionTableConditionItem> decisionItems) {
		this.decisionItems = decisionItems;
	}

	public List<DecisionTableActionItem> getActionItems() {
		return actionItems;
	}

	public void setActionItems(List<DecisionTableActionItem> actionItems) {
		this.actionItems = actionItems;
	}

	public List<DecisionTableValue> getValueTable() {
		return valueTable;
	}

	public void setValueTable(List<DecisionTableValue> valueTable) {
		this.valueTable = valueTable;
	}

	public String getName() {
		return "decisiontable";
	}

	public Object evaluate() {
		for (DecisionTableValue value : valueTable) {

			// 对所有的条件进行检查
			boolean check = true;
			for (DecisionTableConditionItem item : decisionItems) {
				for (String key : value.getValueTable().get(item.getTitle())
						.keySet()) {
					getRuleSession().getContext()
							.put(key,
									value.getValueTable().get(item.getTitle())
											.get(key));
				}
				item.getBooleanExpresion().setRuleSession(getRuleSession());
				if (!item.getBooleanExpresion().evaluate()) {
					check = false;
				}
			}
			// 执行操作
			if (check) {
				for (DecisionTableActionItem item : actionItems) {
					// 设置操作点位符
					Map<String, String> map = value.getValueTable().get(
							item.getTitle());
					for (String key : map.keySet()) {
						getRuleSession().getContext().put(
								key,
								value.getValueTable().get(item.getTitle())
										.get(key));
					}
					item.getActionExpresion().setRuleSession(getRuleSession());
					item.getActionExpresion().evaluate();
				}
			}
		}
		return null;
	}
}
